from ..Qt import QtWidgets, QtCore


class _FileDialog(QtWidgets.QFileDialog):
    def __init__(self, directory=None, caption=None, filters='', parent=None):
        super().__init__(parent=parent)

        self.setOption(QtWidgets.QFileDialog.DontUseNativeDialog, True)

        self.setWindowTitle(caption)
        self.setDirectory(str(directory))
        self.setNameFilters(filters.split(';;'))

        file_tree = self.findChild(QtWidgets.QTreeView, 'treeView')
        file_tree.setSortingEnabled(True)
        for i in range(file_tree.header().count()):
            file_tree.header().setSectionResizeMode(i, 0)
        file_tree.model().sort(0)

        file_list = self.findChild(QtWidgets.QListView, 'listView')
        file_list.model().sort(0)

        line = QtWidgets.QFrame(self)
        line.setFrameShape(line.HLine)
        line.setFrameShadow(line.Sunken)
        self.layout().addWidget(line, self.layout().rowCount(), 0, 1, self.layout().columnCount())


class OpenFileDialog(_FileDialog):
    def __init__(self, directory=None, caption=None, filters='', parent=None):
        super().__init__(directory=directory, caption=caption, filters=filters, parent=parent)

        self.setFileMode(QtWidgets.QFileDialog.ExistingFiles)

        self.checkBox = QtWidgets.QCheckBox(self)
        self.checkBox.setChecked(False)
        self.checkBox.setText('Use existing graph')
        self.checkBox.stateChanged.connect(self.checkbox_state)
        self.layout().addWidget(self.checkBox)

        self.comboBox = QtWidgets.QComboBox(self)
        self.comboBox.setEnabled(False)
        self.layout().addWidget(self.comboBox)

        self.add_to_graph = None

    def set_graphs(self, graphs):
        self.comboBox.blockSignals(True)
        for gid, name in graphs:
            self.comboBox.addItem(name, userData=gid)
        self.comboBox.blockSignals(False)

        if not self.comboBox.count():
            self.comboBox.hide()
            self.checkBox.hide()

    def checkbox_state(self, checked: QtCore.Qt.CheckState):
        if checked == QtCore.Qt.Checked:
            self.comboBox.setEnabled(True)
            self.add_to_graph = self.comboBox.currentData()
        else:
            self.comboBox.setEnabled(False)
            self.add_to_graph = None

    @QtCore.pyqtSlot(int, name='on_comboBox_currentIndexChanged')
    def graph_changed(self, idx: int):
        self.add_to_graph = self.comboBox.itemData(idx)


class SaveDirectoryDialog(_FileDialog):
    def __init__(self, directory=None, filters='', parent=None):
        super().__init__(directory=directory, filters=filters, parent=parent)

        self.setOption(QtWidgets.QFileDialog.DontConfirmOverwrite, False)
        self.setAcceptMode(QtWidgets.QFileDialog.AcceptSave)

        self.label = QtWidgets.QLabel(self)
        self.label.setTextFormat(QtCore.Qt.RichText)
        self.label.setText('Use <b>&lt;label&gt;</b> as placeholder in filename. (e.g. <i>t1_&lt;label&gt;.dat</i>)')
        self.layout().addWidget(self.label, self.layout().rowCount(), 0, 1, self.layout().columnCount())

        self.checkBox = QtWidgets.QCheckBox(self)
        self.checkBox.setChecked(True)
        self.checkBox.setText('Replace spaces with underscore')
        self.layout().addWidget(self.checkBox, self.layout().rowCount(), 0, 1, self.layout().columnCount())

        self.setWindowTitle('Save')
        self.setNameFilters(['All files (*.*)', 'Session file (*.nmr)', 'Text file (*.dat)', 'HDF file (*.h5)', 'Grace files (*.agr)'])
